define([
  "dojo/_base/declare",
  "dojo/dom-class",
  "dojo/on",
  "dijit/form/TimeTextBox"
], function(declare, domClass, on, TimeTextBox) {
  return declare("wagby.TimeTextBox", [TimeTextBox], {
    DEFAULT_TIME_FORMAT: "HH:mm",
    pickerMin: "T00:00:00",
    pickerMax: "T23:59:59",
    timeIncrement: "T00:15:00",
    timeFormat: "",
    timeSelectInput: "",
    enterKeyAction: null,

    buildRendering: function() {
      this.inherited(arguments);
      this.setConstraints();
    },

    postCreate: function() {
      this.inherited(arguments);
      this.setEnterKeyActionEvent();
      this.setErrorStyle();
    },

    serialize: function(val, options) {
      var value = this.inherited(arguments);
      if (!value) {
        return value;
      }
      return value.replace("T", "");
    },

    _setTimeFormatAttr: function(value) {
      if (!(value === "HH" || value === "HH:mm" || value === "HH:mm:ss")) {
        return;
      }
      this.timeFormat = value;
    },

    _setTimeSelectInputAttr: function(value) {
      if (!value || !this.constraints) {
        return;
      }
      var props = /^H:(\d{1,2})\-(\d{1,2})$/.exec(value)
        || /^HM\:(\d{1,2})-(\d{1,2})\:(\d{1,2})$/.exec(value);
      if (props) {
        this.setTimeSelectConstraints(props);
      }
    },

    setTimeSelectConstraints: function(props) {
      this.constraints.pickerMin = this.getPickerTime(props[1]) || this.pickerMin;
      this.constraints.pickerMax = this.getPickerTime(props[2]) || this.pickerMax;
      this.constraints.timeIncrement = this.getTimeIncrementValue(props[3]) || this.timeIncrement;
    },

    getPickerTime: function(hour) {
      if (!hour) {
        return null;
      }
      if (hour === "24") {
        return "T23:59:59";
      }
      return "T" + this.getZeroPaddingValue(hour) + ":00:00";
    },

    getZeroPaddingValue: function(value) {
      return ("0" + value).substr(-2);
    },

    getTimeIncrementValue: function(value) {
      if (!value) {
        return null;
      }
      var timeIncrementValue = value;
      if (timeIncrementValue === "0") {
        timeIncrementValue = "1";
      }
      return "T00:" + this.getZeroPaddingValue(timeIncrementValue) + ":00";
    },

    setConstraints: function() {
      if (!this.timeFormat || this.timeFormat === "") {
        this.timeFormat = this.DEFAULT_TIME_FORMAT;
      }
      if (this.timeFormat === "HH") {
        this.timeIncrement = "T01:00:00";
      }
      this.set("constraints", {
        timePattern: this.timeFormat,
        pickerMin: this.pickerMin,
        pickerMax: this.pickerMax,
        clickableIncrement: this.timeIncrement,
      });
    },

    setEnterKeyActionEvent: function() {
      if (!this.enterKeyAction) {
        return;
      }
      var keydownFunc;
      var that = this;
      if (this.enterKeyAction === "changetab") {
        this.textbox.on("keydown", function(event) {
          event.preventDefault();
          enterKeyHandler.focusNextElement(that, event);
        });
        return;
      } else if (this.enterKeyAction === "ignore") {
        keydownFunc = function(event) {
          if (event.keyCode == 0x0d) { event.preventDefault(); }
        }
      } else if (this.enterKeyAction === "default") {
        keydownFunc = function(event) {
          if (event.keyCode == 0x0d) { doSubmitForm(event.srcElement.form, 'action_Send'); }
        }
      }
      this.textbox.on("keydown", keydownFunc);
    },

    setErrorStyle: function() {
      if (!this.inputError || this.inputError === "false") {
        return;
      }
      domClass.add(this.domNode, "errorfield");
    },

  });
});
